/**
 * Project Name: warehouse
 * File Name: MaterialsService
 * Package Name: com.starcharge.warehouse.service
 * Date: 2020/10/16 10:57
 * Copyright (c) 2020,All Rights Reserved.
 */
package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.boco.nbd.wios.manage.entity.bo.Materials;
import com.boco.nbd.wios.manage.entity.bo.MaterialsEx;
import com.boco.nbd.wios.manage.entity.bo.WarehouseMaterials;
import com.boco.nbd.wios.manage.entity.bo.WarehouseMaterialsCriteria;
import com.boco.nbd.wios.manage.entity.dto.MaterialsQuery;
import com.boco.nbd.wios.manage.entity.dto.MaterialsSaveQuery;
import com.boco.nbd.wios.manage.mapper.extdb.MaterialsMapperEx;
import com.boco.nbd.wios.manage.mapper.extdb.WarehouseMaterialsMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.util.StringUtilsEx;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 物料相关业务处理
 *
 * @author lvpeng
 * @date 2020/10/16 10:57
 */
@Service
public class MaterialsService {

    @Resource
    MaterialsMapperEx materialsMapperEx;

    @Autowired
    CompanyService companyService;

    @Resource
    WarehouseMaterialsMapper warehouseMaterialsMapper;

    /**
     * 新增物料，支持批量新增、修改
     *
     * @param appkey
     * @param materialsSaveQueryList 物料集合对象
     */
    public void saveMaterials(String appkey, List<MaterialsSaveQuery> materialsSaveQueryList, String accountId) {
        //判断appkey是否存在，如果不存在则报错
        if (CollUtil.isEmpty(companyService.getCompanyByAppKey(appkey))) {
            throw new ServiceException("11002", StringUtils.EMPTY);
        }
        // 循环物料集合进行批量入库，如果存在则更新，不存在则新增
        List<Materials> materialsList = new ArrayList<>(1);
        materialsSaveQueryList.forEach(materialsSaveQuery -> {
            Materials materials = new Materials();
            materials.setId(StringUtils.isNoneBlank(materialsSaveQuery.getId()) ? materialsSaveQuery.getId() : StringUtilsEx.getUUID());
            materials.setAppkey(appkey);
            materials.setMaterialsName(materialsSaveQuery.getMaterialsName());
            materials.setMaterialsCode(materialsSaveQuery.getMaterialsCode());
            materials.setRegularModel(materialsSaveQuery.getRegularModel());
            materials.setUnit(materialsSaveQuery.getUnit());
            materials.setType(materialsSaveQuery.getType());
            materials.setStatus(1);
            materials.setCreateAccount(accountId);
            materials.setCreateTime(new Date());
            materials.setModifyAccount(accountId);
            materials.setModifyTime(new Date());
            materialsList.add(materials);
        });
        materialsMapperEx.insertMaterials(materialsList);
    }

    /**
     * 获取物料信息
     *
     * @param materialsQuery
     * @return
     */
    public List<MaterialsEx> qryMaterialsList(MaterialsQuery materialsQuery) {
        List<String> warehouseIds = materialsQuery.splitData(materialsQuery.getWarehouseId());
        List<String> typeList = materialsQuery.splitData(materialsQuery.getTypes());
        return materialsMapperEx.qryMaterialsList(materialsQuery.getAppkey(), warehouseIds, materialsQuery.getMaterialsName(), materialsQuery.getMaterialsCode(), materialsQuery.getRegularModel(), materialsQuery.getType(), materialsQuery.getId(), typeList, materialsQuery.getIsBindWareHouse());
    }


    /**
     * 获取库存数据
     *
     * @param materialsQuery
     * @return
     */
    public List<MaterialsEx> qryMaterialsStockList(MaterialsQuery materialsQuery) {
        List<String> warehouseIds = materialsQuery.splitData(materialsQuery.getWarehouseId());
        List<String> typeList = materialsQuery.splitData(materialsQuery.getTypes());
        return materialsMapperEx.qryMaterialsStockList(materialsQuery.getAppkey(), warehouseIds, materialsQuery.getMaterialsName(), materialsQuery.getMaterialsCode(), materialsQuery.getRegularModel(), materialsQuery.getType(), materialsQuery.getId(), typeList, materialsQuery.getIsBindWareHouse());
    }


    /**
     * 根据条件查询数量
     *
     * @param appkey
     * @param materialsName
     * @param materialsCode
     * @param regularModel
     * @param type
     * @return
     */
    public List<MaterialsEx> qryMaterialsCount(String appkey, String materialsName, String materialsCode, String regularModel, Integer type) {
        return materialsMapperEx.qryMaterialsCount(appkey, materialsName, materialsCode, regularModel, type);
    }

    /**
     * 删除物料
     *
     * @param materialsId
     */
    public void delMaterials(String materialsId) {
        // 判断物料是否已经使用或已经关联仓库
        // 如果关联则提示不能删除
        materialsMapperEx.delMaterialsById(materialsId);
    }


    /**
     * 获取仓库下某物料数据
     *
     * @param warehouseId
     * @param materialsId
     * @return
     */
    public List<WarehouseMaterials> getWarehouseMaterials(String warehouseId, String materialsId) {
        return materialsMapperEx.getWarehouseMaterials(warehouseId, materialsId);
    }


    /**
     * 更新剩余可用数量
     *
     * @param warehouseId
     * @param materialsId
     * @param number
     * @param type        1:新增待用数量 2：减去待用数量
     * @return
     */
    public void updateMaterialsRemainingNumber(String warehouseId, String materialsId, Integer number, Integer type) {
        // 根据仓库Id 及物料ID获取相应数据
        WarehouseMaterialsCriteria warehouseMaterialsCriteria = new WarehouseMaterialsCriteria();
        WarehouseMaterialsCriteria.Criteria criteria = warehouseMaterialsCriteria.createCriteria();
        criteria.andWarehouseIdEqualTo(warehouseId);
        criteria.andMaterialsIdEqualTo(materialsId);
        List<WarehouseMaterials> warehouseMaterialsList = warehouseMaterialsMapper.selectByExample(warehouseMaterialsCriteria);
        if (CollUtil.isNotEmpty(warehouseMaterialsList)) {
            WarehouseMaterials warehouseMaterials = warehouseMaterialsList.get(0);
            // 剩余数量与待用数量比较，如果待用数量大于剩余数量则抛出异常
            if (type == 1 && warehouseMaterials.getRemainingNumber().compareTo(new BigDecimal(number)) < 0) {
                throw new ServiceException("剩余数量不足");
            }
            if (type == 2 && warehouseMaterials.getStayWithNumber().compareTo(new BigDecimal(number)) < 0) {
                throw new ServiceException("待用数量不足");
            }
            if (type == 1) {
                warehouseMaterials.setRemainingNumber((warehouseMaterials.getRemainingNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getRemainingNumber()).subtract(new BigDecimal(number)));
                warehouseMaterials.setStayWithNumber((warehouseMaterials.getStayWithNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getStayWithNumber()).add(new BigDecimal(number)));
            } else if (type == 2) {
                warehouseMaterials.setRemainingNumber((warehouseMaterials.getRemainingNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getRemainingNumber()).add(new BigDecimal(number)));
                warehouseMaterials.setStayWithNumber((warehouseMaterials.getStayWithNumber() == null ? BigDecimal.ZERO : warehouseMaterials.getStayWithNumber()).subtract(new BigDecimal(number)));
            }
            warehouseMaterialsMapper.updateByPrimaryKey(warehouseMaterials);
        } else {
            throw new ServiceException("仓库或物料不存在");
        }
    }

}
