package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.boco.nbd.framework.workflow.entity.exception.FlowException;
import com.boco.nbd.wios.manage.entity.bo.InvoiceCriteria;
import com.boco.nbd.wios.manage.entity.bo.Order;
import com.boco.nbd.wios.manage.entity.bo.UserPayBill;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.entity.settlement.po.Invoice;
import com.boco.nbd.wios.manage.entity.settlement.vo.InvoiceVo;
import com.boco.nbd.wios.manage.mapper.def.InvoiceDao;
import com.boco.nbd.wios.manage.mapper.def.OrderDao;
import com.boco.nbd.wios.manage.mapper.def.UserPayBillDao;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @author licheng
 * @Description OrderInvoiceService
 * @date 2021/7/24 13:41
 * @return
 */
@Service
@Slf4j
public class OrderInvoiceService {

    @Autowired
    private InvoiceDao invoiceDao;
    @Autowired
    private UserPayBillDao userPayBillDao;
    @Autowired
    private InvoiceService invoiceService;

    @Autowired
    private SapRemoteService sapRemoteService;

    @Autowired
    private OrderDao orderDao;

    /**
     * 流程版本批量更新
     * 服务商结算: supplier_receive_invoice_status=1 ;submit_item 空
     * 增项结算：invoice_status=1;submit_item 空
     * 1-服务开票，2-设备开票，两个同时-3
     * 主机厂结算:
     * 1) submit_item=1 or 3 ;package_install_invoice_status=1
     * 2) submit_item=2 or 3 ;package_device_invoice_status=1
     *
     * @param invoiceJson
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void processBatchUpdate(String invoiceJson) {
        try {
            List<Invoice> list = Invoice.jsonHandle(invoiceJson);
            if (CollUtil.isEmpty(list)) {
                throw new FlowException("批量更新数据为空.");
            }
            list.forEach(s -> invoiceHandle(s, false));
        } catch (Exception ex) {
            log.error("批量更新开票信息异常:", ex);
            throw new FlowException("批量更新开票信息异常:" + ex.getMessage());
        }

    }

    /**
     * 票据处理
     *
     * @param invoice
     * @param useSap:是否推动
     */
    private void invoiceHandle(Invoice invoice, boolean useSap) {
        try {
            invoice.setModifyTime(new Date());
            //提交项（1-服务开票，2-设备开票，两个同时-3）
            Integer submititem = invoice.getSubmitItem();

            int i = 0;
            if (submititem == null) {
                //不含税
                if (invoice.getSupplierFeeWithTax() != null) {
                    BigDecimal supplierFee = invoice.getSupplierFeeWithTax();
                    invoice.setSupplierFeeWithTax(invoice.getSupplierTax().add(supplierFee));
                } else {
                    BigDecimal fee = invoice.getFeeWithTax();
                    invoice.setFeeWithTax(invoice.getTax().add(fee));
                }

                i = invoiceDao.updateSupplier(invoice);
                if (i == 0) {
                    invoice.setCreateTime(new Date());
                    i = invoiceDao.insertSelective(invoice);
                }
                //服务商收票后调用
                if (i == 1 && invoice.getSupplierReceiveInvoiceStatus() != null && invoice.getSupplierReceiveInvoiceStatus() == 1) {
                    Order order = orderDao.selectById(invoice.getOrderId());
                    if (order == null) {
                        return;
                    }
                    //wios->sap订单服务商结算数据同步
                    if (useSap) {
                        ThreadUtil.execAsync(() -> sapRemoteService.syncSettlementData(order, invoice));
                    }

                }
                //增项
                if (i == 1 && invoice.getReceiveInvoiceStatus() != null && invoice.getReceiveInvoiceStatus() == 1) {
                    Order order = orderDao.selectById(invoice.getOrderId());
                    if (order == null) {
                        return;
                    }
                    //wios->sap订单增项结算数据同步
                    invoice.setInvoiceType(1);
                    ThreadUtil.execAsync(() -> sapRemoteService.pushNewItemWiosToSap(order, invoice));

                }
            } else {
                if (submititem == 1) {
                    i = invoiceDao.updateInstall(invoice);
                } else if (submititem == 2) {
                    i = invoiceDao.updateDevice(invoice);
                } else if (submititem == 3) {
                    i = invoiceDao.updateByPrimaryKeySelective(invoice);
                }

                if (i == 0) {
                    invoice.setCreateTime(new Date());
                    i = invoiceDao.insertSelective(invoice);
                }
                //套包内安装服务开票后调用订单推送
                if ((submititem == 1 || submititem == 3) && invoice.getPackageInstallInvoiceStatus() != null && invoice.getPackageInstallInvoiceStatus() == 1) {
                    invoice.setSubmitItem(1);
                    Order order = orderDao.selectById(invoice.getOrderId());
                    if (order == null) {
                        return;
                    }
                    if (useSap && OrderStatus.arePushSap(order.getStatus())) {
                        //wios->sap订单推送
                        log.info(order.getId() + ",安装服务开票开始同步SAP");
                        Invoice invoice1 = new Invoice();
                        Order order1 = new Order();
                        BeanUtils.copyProperties(invoice, invoice1);
                        BeanUtils.copyProperties(order, order1);
                        ThreadUtil.execAsync(() -> sapRemoteService.pushOrderOrInvoice(order1, invoice1));
                    }
                }
                if ((submititem == 2 || submititem == 3) && invoice.getPackageDeviceInvoiceStatus() != null && invoice.getPackageDeviceInvoiceStatus() == 1) {
                    invoice.setSubmitItem(2);
                    Order order = orderDao.selectById(invoice.getOrderId());
                    if (order == null) {
                        return;
                    }
                    if (useSap && OrderStatus.arePushSap(order.getStatus())) {
                        //wios->sap订单推送
                        log.info(order.getId() + ",设备开票开始同步SAP");
                        Invoice invoice2 = new Invoice();
                        Order order2 = new Order();
                        BeanUtils.copyProperties(invoice, invoice2);
                        BeanUtils.copyProperties(order, order2);
                        ThreadUtil.execAsync(() -> sapRemoteService.pushOrderOrInvoice(order2, invoice2));
                    }
                }
            }
        } catch (Exception ex) {
            log.error("票据处理异常:", ex);
            throw new FlowException("票据处理异常");
        }
    }

    /**
     * 批量更新
     *
     * @param supplierInvoiceJson
     */
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void batchUpdate(String supplierInvoiceJson) {
        ObjectMapper mapper = new ObjectMapper();
        try {
            List<Invoice> list = mapper.readValue(supplierInvoiceJson, new TypeReference<List<Invoice>>() {
            });
            list.forEach(invoice -> invoiceHandle(invoice, true));
        } catch (JsonProcessingException e) {
            log.error("json转换失败", e);
            throw new ServiceException("转换失败");
        }
    }

    /**
     * 获取发票表详情
     *
     * @param orderId
     * @return
     */
    public Invoice getByOrderId(String orderId) {
        return invoiceDao.selectByPrimaryKey(orderId);
    }

    /**
     * 更新增项开票状态
     */
    public void updateAdditionalByBillId(String billId) {
        if (StrUtil.isNotBlank(billId)) {
            UserPayBill payBill = userPayBillDao.getById(billId);
            if (payBill != null) {
                Invoice record = new Invoice();
                record.setOrderId(payBill.getRefId());
                record.setModifyTime(new Date());
                int i = invoiceDao.updateByPrimaryKeySelective(record);
                if (i == 0) {
                    record.setCreateTime(new Date());
                    invoiceDao.insertSelective(record);
                }
            }
        }
    }

    /**
     * 更新增项开票状态
     */
    public void invoiceUpdater() {
        InvoiceCriteria example = new InvoiceCriteria();
        example.createCriteria().andAdditionalCostInvoiceStatusIsNull();
        List<Invoice> invoices = invoiceDao.selectByExample(example);
        for (Invoice invoice : invoices) {
            UserPayBill entity = new UserPayBill();
            entity.setRefId(invoice.getOrderId());
            List<UserPayBill> list = userPayBillDao.list(entity);
            if (CollUtil.isNotEmpty(list)) {
                UserPayBill payBill = list.get(0);
                try {
                    InvoiceVo invoiceVo = invoiceService.getInvoiceByBillId(payBill.getId());
                    if (invoiceVo != null && invoiceVo.getStatus() > 0) {
                        Invoice record = new Invoice();
                        record.setOrderId(payBill.getRefId());
                        record.setAdditionalCostInvoiceStatus(1);
                        record.setAdditionalCostInvoiceNumber(invoiceVo.getInvoiceNumber());
                        record.setAdditionalCostInvoiceDate(invoiceVo.getInvoiceTime());
                        record.setModifyTime(new Date());
                        invoiceDao.updateByPrimaryKeySelective(record);
                    }
                } catch (Exception e) {
                    log.error("调用三方接口根据工单账单编号查询发票详情异常:", e);
                }

            }
        }
    }
}
