package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.cams.enums.OrderStatus;
import com.boco.nbd.wios.manage.mapper.def.OperateLogDao;
import com.boco.nbd.wios.manage.mapper.def.OrderDao;
import com.boco.nbd.wios.manage.mapper.def.OrderScoreDao;

import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 供应商评分的明细
 *
 * @author yongfu.liu@wanbangauto.com 2020-10-10 14:08:14
 */
@Service
public class OrderScoreService {

    @Autowired
    private OrderScoreDao orderScoreDao;

    @Autowired
    private OperateLogDao operateLogDao;

    @Autowired
    private ScoreSupplierService scoreSupplierService;

    @Autowired
    private OrderDao orderDao;

    /**
     * 根据操作日志判断客户是否已经评价（回访）
     *
     * @param orderId
     * @return
     */
    public boolean isOrderScore(String orderId) {
        return getOrderScoreLog(orderId) != null;
    }

    public OperateLog getOrderScoreLog(String orderId) {
        OperateLogBo operateLogQuery = new OperateLogBo();
        operateLogQuery.setThirdPartyId(orderId);
        operateLogQuery.setName(String.valueOf(OrderStatus.RETURN_VISIT.getType()));
        List<OperateLog> operateLogList = operateLogDao.selectByExample(operateLogQuery);
        return CollectionUtils.isNotEmpty(operateLogList) ? operateLogList.get(0) : null;
    }

    /**
     * 获取工单评分
     * @param orderId
     * @return
     */
    public OrderScoreDTO getOrderScoreByOrderId(String orderId) {
        OrderScoreDTO orderScoreDTO = new OrderScoreDTO();
        orderScoreDTO.setOrderId(orderId);
        OperateLogBo operateLogQuery = new OperateLogBo();
        operateLogQuery.setThirdPartyId(orderId);
        operateLogQuery.setName(OrderStatus.RETURN_VISIT.getType() + "");
        List<OperateLog> operateLogList = operateLogDao.selectByExample(operateLogQuery);
        if (CollUtil.isNotEmpty(operateLogList)) {
            orderScoreDTO.setVisitContent(operateLogList.get(0).getDesc());
            orderScoreDTO.setVisitorId(operateLogList.get(0).getCreateAccount());
            orderScoreDTO.setVisitorName(operateLogList.get(0).getCreateAccountName());
            orderScoreDTO.setVisitTime(operateLogList.get(0).getCreateTime());

            Order order = orderDao.selectById(orderId);
            if (order != null) {
                orderScoreDTO.setFiles(order.getFiles());
                orderScoreDTO.setKnowAfterSalesChannel(order.getKnowAfterSalesChannel());
                orderScoreDTO.setInstallLength(order.getInstallLength());
            }
            orderScoreDTO.setHasScore(Boolean.TRUE);
        } else {
            orderScoreDTO.setVisitContent("");
            SessionInfo user = ProcessUtil.getUserInfo();
            if (user != null) {
                orderScoreDTO.setVisitorId(Integer.valueOf(user.getUserId()));
                orderScoreDTO.setVisitorName(user.getUserName());
                orderScoreDTO.setVisitTime(DateUtil.date());
                orderScoreDTO.setHasScore(Boolean.FALSE);
            }
        }
        orderScoreDTO.setOrderScoreList(this.getListByOrderId(orderId));

        return orderScoreDTO;
    }

    /**
     * 查询明细
     *
     * @param orderId
     * @return
     */
    public List<OrderScore> getListByOrderId(String orderId) {
        List<OrderScore> list = orderScoreDao.getListByOrderId(orderId);
        if (CollUtil.isEmpty(list)) {
            //如果没有记录，则增加相应的客户评分记录，初始化
            orderScoreDao.insertOrderScore(orderId);
            list = orderScoreDao.getListByOrderId(orderId);
        }
        return list;
    }

    /**
     * 单个更新
     *
     * @param orderScore
     */
    public void update(OrderScore orderScore) {
        if (orderScore.getId() == null || orderScore.getId() == 0) {
            throw new ServiceException("没有该条记录的 ID");
        }
        orderScoreDao.update(orderScore);
    }

    /**
     * 通过编码和主表id更新明细表记录
     *
     * @param orderScore
     */
    public void updateByCode(OrderScore orderScore) {
        if (StringUtils.isEmpty(orderScore.getCode()) || orderScore.getOrderId() == null) {
            throw new ServiceException("编码和订单表ID不能为空！");
        }
        orderScoreDao.updateByCode(orderScore);
    }

    /**
     * 评分小于60的订单，用于评分报表
     *
     * @param scoreSupplierId
     * @return
     */
    public List<OrderScoreLessThan60DTO> getListLessThan60(Long scoreSupplierId) {
        List<OrderScoreLessThan60DTO> result = new ArrayList<OrderScoreLessThan60DTO>();
        ScoreSupplierDTO scoreSupplier = scoreSupplierService.selectById(scoreSupplierId);
        List<Order> orderList = orderScoreDao.getOrderListLessThan60(scoreSupplier.getSupplierId(), scoreSupplier.getRegionId(), scoreSupplier.getMonth());

        for (Order order : orderList) {
            List<OrderScore> orderScoreList = orderScoreDao.getListByOrderId(order.getId());
            for (OrderScore orderScore : orderScoreList) {
                if (orderScore.getScore() != null && orderScore.getScore().doubleValue() < 60) {
                    OrderScoreLessThan60DTO orderScoreLessThan60DTO = new OrderScoreLessThan60DTO();
                    orderScoreLessThan60DTO.setOrderId(order.getId());
                    orderScoreLessThan60DTO.setOrderScore(order.getScore());
                    orderScoreLessThan60DTO.setOrderFiles(order.getFiles());
                    orderScoreLessThan60DTO.setCode(orderScore.getCode());
                    orderScoreLessThan60DTO.setName(orderScore.getName());
                    orderScoreLessThan60DTO.setScore(orderScore.getScore());
                    orderScoreLessThan60DTO.setWeight(orderScore.getWeight());
                    result.add(orderScoreLessThan60DTO);
                }
            }
        }
        return result;
    }

}
