package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.framework.common.ResponseMessage2;
import com.boco.nbd.wios.flow.entity.bo.SapStatisticsBO;
import com.boco.nbd.wios.flow.entity.po.OemCascadePO;
import com.boco.nbd.wios.flow.mapper.def.OemCascadeMapper;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.entity.bo.Dictionary;
import com.boco.nbd.wios.manage.entity.dto.MaterialsDTO;
import com.boco.nbd.wios.manage.entity.settlement.po.Invoice;
import com.boco.nbd.wios.manage.entity.settlement.po.SettlementDetailSupplier;
import com.boco.nbd.wios.manage.enums.PushOrderTypeEnum;
import com.boco.nbd.wios.manage.mapper.def.*;
import com.boco.nbd.wios.manage.mapper.extdb.MaterialsMapperEx;
import com.boco.nbd.wios.manage.util.HttpUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: sap系统接口调用
 * @data:
 * @return:
 * @Date: 2022/11/11
 * @Author: yuanlong
 */
@Service
public class SapRemoteService {

    private Logger log = LoggerFactory.getLogger(getClass());

    @Value("${sap.url}")
    private String sapUrl;

    @Value("${sap.username}")
    private String sapUsername;

    @Value("${sap.password}")
    private String sapPassword;

    @Autowired
    OemCascadeMapper oemCascadeMapper;

    @Autowired
    private OemDao oemDao;

    @Autowired
    private SupplierDao supplierDao;

    @Autowired
    private DictionaryDao dictionaryDao;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private SettlementService settlementService;

    @Autowired
    private SupplierContractDao supplierContractDao;
    @Autowired
    private OrderInstallDao orderInstallDao;

    @Autowired
    private OrderInstallItemDao orderInstallItemDao;

    @Autowired
    OemContractItemDao oemContractItemDao;

    @Autowired
    SupplierContractPriceDao supplierContractPriceDao;

    @Autowired
    MaterialsMapperEx materialsMapperEx;

    @Autowired
    OemContractPriceDao oemContractPriceDao;

    private final String SAP_RESULT_TYPE = "S";

    /**
     * 当墙盒名称为CAMS的时候item传两次
     */
    private final String WALL_BOX_NAME_CAMS = "CAMS";

    /**
     * 只有个安装服务商=A的时候才传增项和高附加值材料  =B/C的时候只传安装服务费和墙盒
     */
    private final String PROVIDER_TYPE_A = "A";

    Double getResponseTime(long start, long end) {
        long elapsedTime = end - start;
        double seconds = (double) elapsedTime / 1000000000.0;
        BigDecimal bg = new BigDecimal(seconds).setScale(2, RoundingMode.UP);
        return bg.doubleValue();
    }

    /**
     * @Description: 订单推送(安装审核通过) 包含安装和高附加值
     * @data:[order, sapCode]
     * @return: java.util.Map<java.lang.Integer, java.lang.String>
     * @Date: 2022/9/28
     * @Author: yuanlong
     */
    public ResponseMessage2<SapStatisticsBO> pushOrder(Order order, String sapCode) {
        String orderId = order.getId();
        Oem oem = oemDao.selectById(order.getOemId());
        if (oem == null) {
            log.warn("获取oem失败,订单id:{}", orderId);
            return ResponseMessage2.Failed("获取oem失败,订单id:" + orderId);
        }

        Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
        if (supplier == null) {
            log.warn("获取supplier失败,订单id:{}", orderId);
            return ResponseMessage2.Failed("获取supplier失败,订单id:" + orderId);
        }

        //获取有效合约ID
        List<SupplierContract> supplierContracts = supplierContractDao.selectEffectiveContract(String.valueOf(order.getInstallSupplierId()), null, 2);
        if (CollectionUtils.isEmpty(supplierContracts)) {
            log.warn("找不到生效中的安装服务商对应合约,订单id:{}", orderId);
            return ResponseMessage2.Failed("找不到生效中的安装服务商对应合约,订单id:" + orderId);
        } else {
            SupplierContract supplierContract = supplierContracts.get(0);
            Integer supplierContractId = supplierContract.getId();
            order.setSupplierContractId(supplierContractId);
        }
        //item_type高附加值材料4,增项2
        List<OrderInstallItemVo> installItemVoList = orderInstallItemDao.selectByOrderIdAndType(orderId, 2);
        List<OrderInstallItemVo> highValueAdded = new ArrayList<>(16);
        if (CollectionUtils.isNotEmpty(installItemVoList)) {
            highValueAdded = installItemVoList.stream().filter(s -> s.getItemType() == 4).collect(Collectors.toList());
        }
        //订单安桩审核状态
        String zstatus1 = "X";
        //安装服务费（含税）
        BigDecimal installServiceFeeWithTax = this.getFirstItem(order, highValueAdded).get("installServiceFeeWithTax");
        //服务税
        BigDecimal serviceTax = this.getFirstItem(order, highValueAdded).get("serviceTax");
        //设备价格（含税）
        BigDecimal deviceFeeWithTax = this.getSecondItem(order).get("deviceFeeWithTax");
        //设备税
        BigDecimal deviceTax = this.getSecondItem(order).get("deviceTax");
        //ZBRANDTXT 墙盒品牌描述
        String zbrandtxt = "";
        String zbrandid = "";
        //获取墙盒ID和墙盒编号
        List<OemCascadePO> cascadeById = oemCascadeMapper.getCascadeById(order.getWallboxModel());
        if (CollectionUtils.isNotEmpty(cascadeById) && order.getType().equals(PushOrderTypeEnum.TYPE_INSTALL.getType())) {
            zbrandtxt = cascadeById.get(0).getName();
            String id = cascadeById.get(0).getId();
            zbrandid = id.substring(id.length() - 1, id.length());
        }
        //获取总价
        BigDecimal zpric = this.getZprice(order);
        //产品编码
        String materialsCode = "";
        //设备编号
        String wallboxNo = "";
        //车型
        String vehicleModel = order.getVehicleModel();
        int size = 1;
        //只有墙品牌为CAMS的时候和当前订单为普通订单的时候循环两次
        if (WALL_BOX_NAME_CAMS.equals(zbrandtxt) && order.getType().equals(PushOrderTypeEnum.TYPE_INSTALL.getType())) {
            size = 2;
            if (!StringUtils.isEmpty(sapCode)) {
                //接口提供和修改第二个item的物料编码功能
                materialsCode = sapCode;
            } else {
                materialsCode = order.getInstallItemModel();
            }
        }
        //设备编号
        OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
        if (orderInstall != null) {
            wallboxNo = orderInstall.getWallboxNo();
        }

        List items = new ArrayList(size);
        //服务商类型
        String providerType = supplier.getLevel();
        //服务商编码
        String code = supplier.getCode();

        for (int i = 1; i <= size; i++) {
            //i 0是服务 1设备
            Map<String, Object> item = new HashMap<>(8);
            //添加空参
            item.putAll(getOtherItemParameter());

            item.put("ZNO", orderId);
            //1开始自增
            item.put("ZITEM", i);
            item.put("ZSTATUS1", zstatus1);
            item.put("ZSTATUS2", "");
            //第三方服务商编号
            item.put("LIFNR", code);
            item.put("ZBRANDID", zbrandid);
            item.put("ZBRANDTXT", zbrandtxt);
            //安装服务固定值：HCFWT000001 设备(如果设备需要结算，根据墙盒信息is_settle判断) WarehouseService.getMaterialById(t_order.install_item_model)
            item.put("MATNR", i == 1 ? "HCFWT000001" : materialsCode);
            item.put("KWMENT", 1);
            item.put("NETWR", i == 1 ? installServiceFeeWithTax : deviceFeeWithTax);
            //税额 必须大于0
            item.put("MWSBP", i == 1 ? serviceTax : deviceTax);

            item.put("LGORT", "");
            item.put("ZPRIC", i == 1 ? zpric : "");
            item.put("ZINVNO", "");
            item.put("ZVENDTYPE", providerType);
            item.put("ZCOL92", wallboxNo);
            item.put("ZCOL93", vehicleModel);
            String zocl94 = "";
            String zocl95 = "";
            if (!StringUtils.isEmpty(zbrandtxt)) {
                if ("CAMS".equals(zbrandtxt)) {
                    zocl95 = "安装+销售";
                } else {
                    zocl95 = "安装";
                }
            }
            if (!StringUtils.isEmpty(order.getPackType())) {
                List<OemCascadeBo> oemCascadeBos = orderInstallItemDao.selectPack(order.getPackType());
                if (CollectionUtils.isNotEmpty(oemCascadeBos)) {
                    zocl94 = oemCascadeBos.get(0).getName();
                }
            }
            item.put("ZCOL94", zocl94);
            item.put("ZCOL95", zocl95);
            items.add(item);
        }
        //添加高附加值,服务商为A的时候传
        if (CollectionUtils.isNotEmpty(highValueAdded) && PROVIDER_TYPE_A.equals(providerType)) {
            Map<String, String> map = new HashMap<>(8);
            map.put("zbrandid", zbrandid);
            map.put("wallboxNo", wallboxNo);
            map.put("zbrandtxt", zbrandtxt);
            this.addHighValueToItem(order, items, highValueAdded, supplier, map);
        }
        return pushOrderToSap(items, this.getHead(order, oem, null), orderId);
    }


    /**
     * @Description: 提取空参传输值
     * @data:[]
     * @return: java.util.Map<java.lang.String, java.lang.Object>
     * @Date: 2022/11/11
     * @Author: yuanlong
     */
    private Map<String, Object> getOtherItemParameter() {
        Map<String, Object> item = new HashMap<>(100);
        item.put("ZCOL01", "");
        item.put("ZCOL02", "");
        item.put("ZCOL03", "");
        item.put("ZCOL04", "");
        item.put("ZCOL05", "");
        item.put("ZCOL06", "");
        item.put("ZCOL07", "");
        item.put("ZCOL08", "");
        item.put("ZCOL09", "");
        item.put("ZCOL10", "");
        item.put("ZCOL11", "");
        item.put("ZCOL12", "");
        item.put("ZCOL13", "");
        item.put("ZCOL14", "");
        item.put("ZCOL15", "");
        item.put("ZCOL16", "");
        item.put("ZCOL17", "");
        item.put("ZCOL18", "");
        item.put("ZCOL19", "");
        item.put("ZCOL20", "");
        item.put("ZCOL21", "");
        item.put("ZCOL22", "");
        item.put("ZCOL23", "");
        item.put("ZCOL24", "");
        item.put("ZCOL25", "");
        item.put("ZCOL26", "");
        item.put("ZCOL27", "");
        item.put("ZCOL28", "");
        item.put("ZCOL29", "");
        item.put("ZCOL30", "");
        item.put("ZCOL91", "");
        item.put("ZCOL96", "");
        return item;
    }

    /**
     * @Description: 增项推送到Sap
     * @data:[order]
     * @return: java.util.Map<java.lang.Integer, java.lang.String>
     * @Date: 2022/9/28
     * @Author: yuanlong
     */
    public ResponseMessage2<SapStatisticsBO> pushAddNewItems(Order order, Map<String, Object> changeHeadMap, Map<String, Object> changeItemMap) {
        String orderId = order.getId();
        Oem oem = oemDao.selectById(order.getOemId());
        if (oem == null) {
            log.warn("获取oem失败,订单id:{}", orderId);
            return ResponseMessage2.Failed("获取oem失败,订单id:" + orderId);
        }
        Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());

        List<OrderInstallItemVo> increaseProject = new ArrayList<>(16);
        List<OrderInstallItemVo> installItemVoList = orderInstallItemDao.selectByOrderIdAndType(orderId, 2);
        if (CollectionUtils.isNotEmpty(installItemVoList)) {
            //item_type高附加值材料4,增项2
            increaseProject = installItemVoList.stream().filter(s -> s.getItemType() == 2).collect(Collectors.toList());
        }
        if (CollectionUtils.isNotEmpty(increaseProject)) {
            return this.addNewItems(this.getHead(order, oem, changeHeadMap), order, increaseProject, supplier, changeItemMap);
        } else {
            return ResponseMessage2.Success();
        }
    }


    /**
     * @Description: 第一个item的税额和总金额（含税）
     * @data:[order, highValueAdded]
     * @return: java.util.Map<java.lang.String, java.math.BigDecimal>
     * @Date: 2022/9/28
     * @Author: yuanlong
     */
    public Map<String, BigDecimal> getFirstItem(Order order, List<OrderInstallItemVo> highValueAdded) {
        BigDecimal installServiceFeeWithTax = BigDecimal.ZERO;
        BigDecimal serviceTax = BigDecimal.ZERO;
        Map<String, BigDecimal> map = new HashMap<>(8);
        List<OemContractPriceVo> contractPricesOfOem = oemContractPriceDao.selectByContractId(order.getContractId());
        if (CollectionUtils.isNotEmpty(contractPricesOfOem)) {
            for (OemContractPriceVo s :
                    contractPricesOfOem) {
                if (s.getPackType().equals(order.getPackType())) {
                    BigDecimal ratePrice = s.getInstallationRatePrice();
                    BigDecimal taxRate = s.getTaxRate();
                    BigDecimal taxRatePercent = taxRate.divide(BigDecimal.valueOf(100.0), 2, RoundingMode.UP);
                    BigDecimal taxRatePercentAdd = taxRatePercent.add(BigDecimal.valueOf(1));
                    if (CollectionUtils.isNotEmpty(highValueAdded)) {
                        installServiceFeeWithTax = s.getInstallationRatePrice();
                        //如果包含高附加值，总金额减去所有高附加值的总金额
                        BigDecimal sumHighPrice = highValueAdded.stream().map(OrderInstallItemVo::getRatePrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                        installServiceFeeWithTax = installServiceFeeWithTax.subtract(sumHighPrice);
                        serviceTax = installServiceFeeWithTax.divide(taxRatePercentAdd, 2).multiply(taxRatePercent).setScale(2, RoundingMode.UP);
                    } else {
                        installServiceFeeWithTax = ratePrice;
                        serviceTax = ratePrice.divide(taxRatePercentAdd, 2).multiply(taxRatePercent).setScale(2, RoundingMode.UP);
                    }
                    break;
                }
            }
        }
        map.put("installServiceFeeWithTax", installServiceFeeWithTax);
        map.put("serviceTax", serviceTax);
        return map;
    }


    /**
     * @Description: 获取第二个item的税额和总金额（含税）
     * @data:[order]
     * @return: java.util.Map<java.lang.String, java.math.BigDecimal>
     * @Date: 2022/9/28
     * @Author: yuanlong
     */
    public Map<String, BigDecimal> getSecondItem(Order order) {
        BigDecimal deviceFeeWithTax = BigDecimal.ZERO;
        BigDecimal deviceTax = BigDecimal.ZERO;
        Map<String, BigDecimal> map = new HashMap<>(8);
        //第二个item的数额和总价
        List<OemContractItemVo> contractItems = oemContractItemDao.selectByContractId(order.getContractId());
        if (CollectionUtils.isNotEmpty(contractItems)) {
            contractItems.forEach(s -> {
                MaterialsDTO dto = warehouseService.getMaterialById(s.getItemId());
                if (dto != null) {
                    s.setItemCode(dto.getMaterialsCode());
                    s.setItemModel(dto.getRegularModel());
                    s.setItemName(dto.getMaterialsName());
                }
            });
            for (OemContractItemVo s :
                    contractItems) {
                if (s.getItemCode().equals(order.getInstallItemModel())) {
                    deviceFeeWithTax = s.getRatePrice();
                    BigDecimal fee = s.getRatePrice();
                    BigDecimal taxRate = s.getTaxRate();
                    BigDecimal taxRatePercent = taxRate.divide(BigDecimal.valueOf(100.0), 2, RoundingMode.UP);
                    BigDecimal taxRatePercentAdd = taxRatePercent.add(BigDecimal.valueOf(1));
                    deviceTax = fee.divide(taxRatePercentAdd, 2).multiply(taxRatePercent).setScale(2, RoundingMode.UP);
                    break;
                }
            }
        }
        map.put("deviceFeeWithTax", deviceFeeWithTax);
        map.put("deviceTax", deviceTax);
        return map;
    }

    /**
     * @Description: 获取第一个item的总价
     * @data:[order]
     * @return: java.math.BigDecimal
     * @Date: 2022/9/28
     * @Author: yuanlong
     */
    BigDecimal getZprice(Order order) {
        BigDecimal zpric = BigDecimal.ZERO;
        String q = order.getInstallItemModel();
        q = q.replaceAll("\r|\n", "");
        List<String> list = materialsMapperEx.selectIdForPrice(q);
        List<SupplierContractPriceVo> contractPrices = supplierContractPriceDao.selectByContractId(order.getSupplierContractId());
        if (CollectionUtils.isNotEmpty(list) && CollectionUtils.isNotEmpty(contractPrices)) {
            for (SupplierContractPriceVo vo : contractPrices) {
                if (!StringUtils.isEmpty(vo.getPackType())
                        && vo.getDeviceModel().equals(list.get(0))
                        && vo.getPackType().equals(order.getPackType())) {
                    zpric = vo.getInstallationRatePrice();
                    break;
                }
            }
        }
        return zpric;
    }


    Map<String, Object> getHead(Order order, Oem oem, Map<String, Object> changeHeadMap) {
        //订单安桩审核状态
        String zstatus1 = "X";
        Map<String, Object> isHead = new HashMap<>(8);
        isHead.put("ZNO", order.getId());
        isHead.put("ZSTATUS1", zstatus1);
        isHead.put("ZSTATUS2", "");
        isHead.put("BUKRS", "1000");
        isHead.put("VKBUR", "101");
        //结算客户编号 测试1000，主机厂编码
        isHead.put("KUNNR", oem.getCode());
        isHead.put("ZDATE1", order.getInstallApproveTime() == null ? "" : DateUtil.format(order.getInstallApproveTime(), "yyyy-MM-dd"));
        isHead.put("ZDATE2", "");
        isHead.put("ZSHIPNO", order.getShipno());
        isHead.put("ZCOL81", "");
        //安装编号
        isHead.put("ZCOL82", order.getOutOrderId());
        isHead.put("ZCOL83", "");
        isHead.put("ZCOL84", "");
        isHead.put("ZCOL85", "");
        isHead.put("ZCOL86", "");

        //替换原有head值
        if (changeHeadMap != null) {
            isHead.putAll(changeHeadMap);
        }
        return isHead;
    }


    /**
     * @Description: 推送订单到sap
     * @data:[items, order, oem]
     * @return: java.util.Map<java.lang.Integer, java.lang.String>
     * @Date: 2022/9/28
     * @Author: yuanlong
     */
    ResponseMessage2<SapStatisticsBO> pushOrderToSap(List items, Map<String, Object> isHead, String orderId) {
        Map<String, Object> param = new HashMap<>(8);
        param.put("IS_HEAD", isHead);
        Map<String, Object> itItem = new HashMap<>(8);
        itItem.put("item", items);
        param.put("IT_ITEM", itItem);
        String basicAuth = HttpUtil.buildBasicAuth(sapUsername, sapPassword, Charset.forName(CamsConstant.UTF_8));
        Map<String, String> headers = new HashMap<>(8);
        headers.put("Authorization", basicAuth);
        headers.put("Content-Type", "application/json");
        String url = sapUrl + "/http/post/zfmsd003";
        log.info("订单推送(安装审核通过)[{}] pushOrder请求参数：{}", orderId, JSONObject.toJSONString(param));
        //接口调用时间统计
        Long start = System.nanoTime();
        Date sapCallTime = Date.from(LocalDateTime.now().atZone(ZoneId.systemDefault()).toInstant());
        String response = HttpUtil.doPost(url, param, headers);
        Long end = System.nanoTime();
        Double responseTime = getResponseTime(start, end);
        if (StringUtils.isNotBlank(response)) {
            try {
                JSONObject resp = JSONObject.parseObject(response);
                String retCode = resp.getString("ZTYPE");
                if (SAP_RESULT_TYPE.equals(retCode)) {
                    log.info("订单:{},推送sap成功", orderId);
                    SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(true, url, response, responseTime, sapCallTime, orderId);
                    return ResponseMessage2.Success2(sapStatisticsBO);
                } else {
                    String message = resp.getString("ZMESG");
                    log.error(message);
                    SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(false, url, response, responseTime, sapCallTime, orderId);
                    return ResponseMessage2.Create(-1, message, sapStatisticsBO);
                }
            } catch (Exception e) {
            }
        }
        log.info("订单推送失败(安装审核通过)[{}] pushOrder请求结果：{}", orderId, response);
        sendWarningEmail(orderId, response, "安装审核通过");
        SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(false, sapUrl, response, responseTime, sapCallTime, orderId);
        return ResponseMessage2.Create(-1, "订单推送失败", sapStatisticsBO);
    }


    /**
     * @Description:传输高附加值
     * @data:[order, items, highValueAdded, supplier, map]
     * @return: void
     * @Date: 2022/11/11
     * @Author: yuanlong
     */
    public void addHighValueToItem(Order order, List items, List<OrderInstallItemVo> highValueAdded, Supplier supplier, Map<String, String> map) {
        String orderId = order.getId();
        if (!CollectionUtils.isEmpty(highValueAdded)) {
            int itemCount = items.size();
            for (int i = 0; i < highValueAdded.size(); i++) {
                itemCount++;
                Map<String, Object> item = new HashMap<>(100);
                //添加空参
                item.putAll(getOtherItemParameter());

                item.put("ZNO", orderId);
                item.put("ZITEM", itemCount);
                item.put("ZSTATUS1", "X");
                item.put("ZSTATUS2", "");
                //第三方服务商编号
                item.put("LIFNR", supplier.getCode());
                if (order.getType().equals(PushOrderTypeEnum.TYPE_INSTALL.getType())) {
                    item.put("ZBRANDID", map.get("zbrandid"));
                    item.put("ZBRANDTXT", map.get("zbrandtxt"));
                }
                //高附加值的物料编码 itemid 去关联t_materials
                List<String> list = materialsMapperEx.selectMaterialsCode(highValueAdded.get(i).getItemId());
                String matnr = "";
                if (CollectionUtils.isNotEmpty(list)) {
                    matnr = list.get(0);
                }
                item.put("MATNR", matnr);
                item.put("KWMENT", highValueAdded.get(i).getQunatity());
                //结算总金额
                BigDecimal ratePrice = highValueAdded.get(i).getRatePrice();
                item.put("NETWR", ratePrice);
                //税额 必须大于0
                BigDecimal taxRate = highValueAdded.get(i).getTaxRate();
                BigDecimal taxRatePercent = taxRate.divide(BigDecimal.valueOf(100.0), 2, RoundingMode.UP);
                BigDecimal taxRatePercentAdd = taxRatePercent.add(BigDecimal.valueOf(1));
                BigDecimal tax = ratePrice.divide(taxRatePercentAdd, 2).multiply(taxRatePercent).setScale(2, RoundingMode.UP);
                item.put("MWSBP", tax);
                item.put("LGORT", "");
                item.put("ZPRIC", "");
                item.put("ZINVNO", "");
                item.put("ZVENDTYPE", supplier.getLevel());
                item.put("ZCOL92", map.get("wallboxNo"));
                item.put("ZCOL93", "");
                item.put("ZCOL94", "");
                item.put("ZCOL95", "");
                items.add(item);
            }
        }
    }

    /**
     * @Description:传输增项
     * @data:[]
     * @return: void
     * @Date: 2022/9/13
     * @Author: yuanlong
     */
    public ResponseMessage2<SapStatisticsBO> addNewItems(Map<String, Object> isHead, Order order, List<OrderInstallItemVo> increaseProject, Supplier supplier, Map<String, Object> changeItemMap) {
        String orderId = "Z" + order.getId();
        isHead.put("ZNO", orderId);
        List<Map<String, Object>> items = new ArrayList();
        int j = 1;
        if (CollectionUtils.isNotEmpty(increaseProject)) {
            for (int i = 0; i < increaseProject.size(); i++) {
                //i 0是服务 1设备
                Map<String, Object> item = new HashMap<>(8);
                //添加空参
                item.putAll(getOtherItemParameter());

                item.put("ZNO", orderId);
                //1开始自增
                item.put("ZITEM", j);
                item.put("ZSTATUS1", "X");
                item.put("ZSTATUS2", "");
                //第三方服务商编号
                item.put("LIFNR", supplier.getCode());
                item.put("ZBRANDID", "");
                item.put("ZBRANDTXT", "");
                //--------------------------------------新增------------------------
                //高附加值的物料编码 itemid 去关联t_materials
                List<String> list = materialsMapperEx.selectMaterialsCode(increaseProject.get(i).getItemId());
                String matnr = "";
                if (CollectionUtils.isNotEmpty(list)) {
                    matnr = list.get(0);
                }
                item.put("MATNR", matnr);
                item.put("KWMENT", increaseProject.get(i).getQunatity());
                item.put("NETWR", increaseProject.get(i).getFee());
                //税额 必须大于0
                BigDecimal fee = increaseProject.get(i).getFee();
                BigDecimal taxRate = increaseProject.get(i).getTaxRate();
                BigDecimal taxRatePercent = taxRate.divide(BigDecimal.valueOf(100.0), 2, RoundingMode.UP);
                BigDecimal taxRatePercentAdd = taxRatePercent.add(BigDecimal.valueOf(1));
                BigDecimal tax = fee.divide(taxRatePercentAdd, 2).multiply(taxRatePercent).setScale(2, RoundingMode.UP);
                item.put("MWSBP", tax);
                item.put("LGORT", "");
                item.put("ZPRIC", "");
                item.put("ZINVNO", "");
                item.put("ZVENDTYPE", supplier.getLevel());

                item.put("ZCOL92", "");
                item.put("ZCOL93", "");
                item.put("ZCOL94", "");
                item.put("ZCOL95", "");
                //替换原有head值
                if (changeItemMap != null) {
                    item.putAll(changeItemMap);
                }
                items.add(item);
                ++j;
            }
        }
        return pushOrderToSap(items, isHead, orderId);
    }


    public ResponseMessage2<SapStatisticsBO> pushNewItemWiosToSap(Order order, Invoice invoice) {
        if (invoice.getInvoiceType() == 1) {
            //head修改
            Map<String, Object> mapForHead = new HashMap<>(2);
            mapForHead.put("ZCOL81", "X");
            //开票日期
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            mapForHead.put("ZDATE2", simpleDateFormat.format(invoice.getInvoiceDate()));
            //item修改
            Map<String, Object> mapForItem = new HashMap<>(2);
            mapForItem.put("ZCOL91", "X");
            //发票号码
            mapForItem.put("ZINVNO", invoice.getInvoiceNumber());
            return pushAddNewItems(order, mapForHead, mapForItem);
        } else {
            return ResponseMessage2.Failed("类型错误");
        }
    }


    /**
     * 订单推送(更新开票状态)
     */
    public synchronized ResponseMessage2<SapStatisticsBO> pushOrderOrInvoice(Order order, Invoice invoice) {
        String orderId = order.getId();
        Map<String, Object> isHead = new HashMap<>(8);
        isHead.put("ZNO", orderId);
        //订单安桩审核状态
        isHead.put("ZSTATUS1", "X");
        //安装开票日期
        String zdate2 = "";
        //服务开票状态
        String packageInstallInvoiceStatus = "";
        //设备开票状态
        String packageDeviceInvoiceStatus = "";

        //服务商合同约定的安装价格
        BigDecimal zpric = BigDecimal.ZERO;
        //ZBRANDTXT 墙盒品牌描述
        String zbrandtxt = "";
        String zbrandid = "";
        if (StringUtils.isBlank(order.getInstallWallboxModel())) {
            order.setInstallWallboxModel(order.getWallboxModel());
        }
        Dictionary dictionary = null;
        if (StringUtils.isNotBlank(order.getInstallWallboxModel())) {
            List<Dictionary> dictionaryList = dictionaryDao.selectValidByValue("wallbox", order.getInstallWallboxModel());
            if (CollectionUtils.isNotEmpty(dictionaryList)) {
                dictionary = dictionaryList.get(0);
            }
        }
        if (dictionary != null) {
            zbrandtxt = dictionary.getName();
            zbrandid = dictionary.getValue();
        }
        //产品编码
        String materialsCode = "";

        BigDecimal tax = BigDecimal.ZERO;
        BigDecimal feeWithTax = BigDecimal.ZERO;
        String invoiceNumber = "";

        //提交项（1-服务开票，2-设备开票）
        int submititem = invoice.getSubmitItem();
        if (submititem == 1 && invoice.getPackageInstallInvoiceStatus() == 1) {
            packageInstallInvoiceStatus = "X";
            tax = invoice.getPackageInstallTax();
            feeWithTax = invoice.getPackageInstallFeeWithTax();
            invoiceNumber = invoice.getPackageInstallInvoiceNumber();
            materialsCode = "HCFWT000001";
            zdate2 = invoice.getPackageInstallInvoiceDate() == null ? "" : DateUtil.format(invoice.getPackageInstallInvoiceDate(), "yyyy-MM-dd");

            Integer supplierContractId = order.getSupplierContractId();
            if (supplierContractId == null) {
                return ResponseMessage2.Failed("服务商ID不存在,订单id：" + orderId);
            }

            SupplierContract contract = supplierContractDao.selectById(supplierContractId);
            if (contract == null) {
                return ResponseMessage2.Failed("服务商合同信息不存在,订单id：" + orderId);
            }

            SettlementDetailSupplier settlementDetailSupplier = settlementService.getSettlementDetailSupplier(order, contract);

            if (settlementDetailSupplier != null) {
                zpric = settlementDetailSupplier.getInstallFeeWithTax().setScale(2, RoundingMode.HALF_DOWN);
            }
        }
        if (submititem == 2 && invoice.getPackageDeviceInvoiceStatus() == 1) {
            packageDeviceInvoiceStatus = "X";
            tax = invoice.getPackageDeviceTax();
            feeWithTax = invoice.getPackageDeviceFeeWithTax();
            invoiceNumber = invoice.getPackageDeviceInvoiceNumber();
            zdate2 = invoice.getPackageDeviceInvoiceDate() == null ? "" : DateUtil.format(invoice.getPackageDeviceInvoiceDate(), "yyyy-MM-dd");

            if (dictionary != null && dictionary.getIsSettle() == 1) {
                if (StringUtils.isBlank(order.getInstallItemModel())) {
                    OrderInstall orderInstall = orderInstallDao.selectByOrderId(orderId);
                    if (orderInstall != null) {
                        order.setInstallItemModel(orderInstall.getWallboxItemId());
                    }
                }
                MaterialsDTO materialsDTO = warehouseService.getMaterialById(order.getInstallItemModel());
                if (materialsDTO == null) {
                    log.warn("获取materialsDTO失败,订单id:{}", orderId);
                    return ResponseMessage2.Failed("获取materialsDTO失败,订单id：" + orderId);
                }
                //物料编码
                materialsCode = materialsDTO.getMaterialsCode();
            }
        }
        isHead.put("ZSTATUS2", packageInstallInvoiceStatus);
        isHead.put("BUKRS", "1000");
        isHead.put("VKBUR", "101");
        Oem oem = oemDao.selectById(order.getOemId());
        if (oem == null) {
            log.warn("获取oem失败,订单id:{}", orderId);
            return ResponseMessage2.Failed("获取oem失败,订单id:" + orderId);
        }
        //结算客户编号 测试1000
        isHead.put("KUNNR", oem.getCode());
        isHead.put("ZDATE1", order.getInstallApproveTime() == null ? "" : DateUtil.format(order.getInstallApproveTime(), "yyyy-MM-dd"));
        isHead.put("ZDATE2", zdate2);
        isHead.put("ZSHIPNO", order.getShipno());
        isHead.put("ZCOL81", packageDeviceInvoiceStatus);
        isHead.put("ZCOL82", "");
        isHead.put("ZCOL83", "");
        isHead.put("ZCOL84", "");
        isHead.put("ZCOL85", "");
        isHead.put("ZCOL86", "");

        Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
        if (supplier == null) {
            log.warn("获取supplier失败,订单id:{}", orderId);
            return ResponseMessage2.Failed("获取supplier失败,订单id:" + orderId);
        }
        List items = new ArrayList(1);
        Map<String, Object> item = new HashMap<>(8);
        //添加空参
        item.putAll(getOtherItemParameter());

        item.put("ZNO", orderId);

        item.put("ZITEM", submititem);
        item.put("ZSTATUS1", "X");
        item.put("ZSTATUS2", packageInstallInvoiceStatus);
        //第三方服务商编号
        item.put("LIFNR", supplier.getCode());
        item.put("ZBRANDID", zbrandid);
        item.put("ZBRANDTXT", zbrandtxt);
        item.put("MATNR", materialsCode);
        item.put("KWMENT", 1);
        item.put("NETWR", feeWithTax);
        //税额 必须大于0
        item.put("MWSBP", tax);
        item.put("LGORT", "");

        item.put("ZPRIC", zpric);

        item.put("ZINVNO", invoiceNumber);
        item.put("ZVENDTYPE", supplier.getLevel());

        item.put("ZCOL91", packageDeviceInvoiceStatus);
        item.put("ZCOL92", "");
        item.put("ZCOL93", "");
        item.put("ZCOL94", "");
        item.put("ZCOL95", "");

        items.add(item);
        Map<String, Object> param = new HashMap<>(8);
        param.put("IS_HEAD", isHead);
        Map<String, Object> itItem = new HashMap<>(8);
        itItem.put("item", items);
        param.put("IT_ITEM", itItem);

        String basicAuth = HttpUtil.buildBasicAuth(sapUsername, sapPassword, Charset.forName(CamsConstant.UTF_8));
        Map<String, String> headers = new HashMap<>(8);
        headers.put("Authorization", basicAuth);
        headers.put("Content-Type", "application/json");
        String url = sapUrl + "/http/post/zfmsd003";

        log.info("订单推送(更新开票状态{})[{}] pushOrderOrInvoice请求参数：{}", submititem, orderId, JSONObject.toJSONString(param));
        //接口调用时间统计
        Long start = System.nanoTime();
        Date sapCallTime = Date.from(LocalDateTime.now().atZone(ZoneId.systemDefault()).toInstant());
        String response = HttpUtil.doPost(url, param, headers);
        Long end = System.nanoTime();
        Double responseTime = getResponseTime(start, end);
        if (StringUtils.isNotBlank(response)) {
            try {
                JSONObject resp = JSONObject.parseObject(response);
                String retCode = resp.getString("ZTYPE");
                if (SAP_RESULT_TYPE.equals(retCode)) {
                    SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(true, url, response, responseTime, sapCallTime, orderId);
                    return ResponseMessage2.Success2(sapStatisticsBO);
                }
            } catch (Exception e) {
            }
        }
        log.info("订单推送失败(更新开票状态{})[{}] pushOrderOrInvoice请求结果：{}", submititem, orderId, response);
        sendWarningEmail(orderId, response, "更新开票状态");
        SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(false, url, response, responseTime, sapCallTime, orderId);
        return ResponseMessage2.Create(-1, "订单Id：" + orderId + "请求sap接口异常", sapStatisticsBO);
    }

    /**
     * 同步订单服务商结算数据
     */
    public synchronized ResponseMessage2<SapStatisticsBO> syncSettlementData(Order order, Invoice invoice) {
        String orderId = order.getId();
        String url = sapUrl + "/http/post/zfmsd004";
        Map<String, Object> paramMap = new HashMap<>(8);
        Map<String, Object> param = new HashMap<>(8);
        param.put("ZNO", orderId);

        Supplier supplier = supplierDao.selectById(order.getInstallSupplierId());
        param.put("LIFNR", supplier.getCode());
        param.put("ZDATE", DateUtil.format(invoice.getSupplierInvoiceDate(), "yyyy-MM-dd"));

        //税额
        BigDecimal supplierTax = invoice.getSupplierTax();
        //安装付款金额（不含税）
        BigDecimal supplierFee = invoice.getSupplierFeeWithTax().subtract(supplierTax);
        //税率
        param.put("ZTAX1", 0.09);
        param.put("ZTAX2", supplierTax);
        param.put("ZNETPR1", supplierFee);
        param.put("ZINVNO1", invoice.getSupplierInvoiceNumber());
        List items = new ArrayList();
        items.add(param);
        Map<String, Object> item = new HashMap<>(8);
        item.put("item", items);
        paramMap.put("LT_INT", item);

        String basicAuth = HttpUtil.buildBasicAuth(sapUsername, sapPassword, Charset.forName(CamsConstant.UTF_8));
        Map<String, String> headers = new HashMap<>(8);
        headers.put("Authorization", basicAuth);
        headers.put("Content-Type", "application/json");
        log.info("同步订单服务商结算数据[{}] syncSettlementData请求参数：{}", orderId, JSONObject.toJSONString(paramMap));
        //接口调用时间统计
        Long start = System.nanoTime();
        Date sapCallTime = Date.from(LocalDateTime.now().atZone(ZoneId.systemDefault()).toInstant());
        String response = HttpUtil.doPost(url, paramMap, headers);
        Long end = System.nanoTime();
        Double responseTime = getResponseTime(start, end);
        if (StringUtils.isNotBlank(response)) {
            try {
                JSONObject resp = JSONObject.parseObject(response);
                String retCode = resp.getJSONObject("ZFMSD004_RESPONSE").getJSONObject("ET_OUT").getJSONObject("item").getString("ZMESSAGE");
                if (SAP_RESULT_TYPE.equals(retCode)) {
                    SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(true, url, response, responseTime, sapCallTime, orderId);
                    return ResponseMessage2.Success2(sapStatisticsBO);
                }
            } catch (Exception e) {
            }
        }
        log.info("同步订单服务商结算数据[{}] syncSettlementData请求结果：{}", orderId, response);
        sendWarningEmail(orderId, response, "同步订单服务商结算数据");
        SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(false, url, response, responseTime, sapCallTime, orderId);
        return ResponseMessage2.Create(-1, "订单Id：" + orderId + ",同步订单服务商结算数据请求sap接口失败", sapStatisticsBO);
    }

    /**
     * 校验服务商录入
     *
     * @param code 服务商编码
     */
    public ResponseMessage2<SapStatisticsBO> verifyServiceProvider(String code) {
        String url = sapUrl + "/http/post_cams/zfmmm007";
        Map<String, Object> paramMap = new HashMap<>(8);
        Map<String, Object> lsinput = new HashMap<>(8);
        lsinput.put("ZCJRQ", "");
        lsinput.put("ZJSRQ", "");
        lsinput.put("ZWBXT", "WIOS");
        paramMap.put("LS_INPUT", lsinput);
        Map<String, Object> ltStcd5 = new HashMap<>(8);
        List items = new ArrayList();
        Map<String, Object> stcd5 = new HashMap<>(8);
        stcd5.put("STCD5", code);
        items.add(stcd5);
        ltStcd5.put("item", items);
        paramMap.put("LT_STCD5", ltStcd5);
        String basicAuth = HttpUtil.buildBasicAuth(sapUsername, sapPassword, Charset.forName(CamsConstant.UTF_8));
        Map<String, String> headers = new HashMap<>(8);
        headers.put("Authorization", basicAuth);
        headers.put("Content-Type", "application/json");
        log.info("校验服务商录入 syncSettlementData请求参数：{}", JSONObject.toJSONString(paramMap));
        //接口调用时间统计
        Long start = System.nanoTime();
        Date sapCallTime = Date.from(LocalDateTime.now().atZone(ZoneId.systemDefault()).toInstant());
        String response = HttpUtil.doPost(url, paramMap, headers);
        Long end = System.nanoTime();
        Double responseTime = getResponseTime(start, end);

        log.info("校验服务商录入 syncSettlementData请求结果：{}", response);
        if (StringUtils.isNotBlank(response)) {
            try {
                JSONObject resp = JSONObject.parseObject(response);
                String retCode = resp.getJSONObject("ZFMMM007_Response").getJSONObject("LT_MESG").getJSONObject("item").getString("ZTYPE");
                if (SAP_RESULT_TYPE.equals(retCode)) {
                    SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(true, url, response, responseTime, sapCallTime, code);
                    return ResponseMessage2.Success2(sapStatisticsBO);
                }
            } catch (Exception e) {
                SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(false, url, response, responseTime, sapCallTime, code);
                return ResponseMessage2.Create(-1, "服务商编号：" + code + "不存在", sapStatisticsBO);
            }
        }
        SapStatisticsBO sapStatisticsBO = new SapStatisticsBO(false, url, response, responseTime, sapCallTime, code);
        return ResponseMessage2.Create(-1, "服务商编号：" + code + "，请求sap接口异常", sapStatisticsBO);
    }


    private static List<String> mailList = new ArrayList<>();

    static {
        mailList.add("jianfeng.gu@camsnetec.com");
        mailList.add("juan.song@camsnetec.com");
        mailList.add("liuhong.fan@camsnetec.com");
        mailList.add("jiaxin.tang@camsnetec.com");
        mailList.add("yijie.fang@camsnetec.com");
        mailList.add("wei.wang@camsnetec.com");
    }

    @Autowired
    private PushService pushService;

    private void sendWarningEmail(String orderId, String response, String title) {
        try {
            mailList.stream().forEach(email -> {
                pushService.sendEmailAsync(email, "SAP订单推送失败(" + title + ")",
                        "<p>您好：</p>" + "<p>订单号为" + orderId + "的订单，" + title + "时同步SAP失败，请及时查看</p><p>SAP返回值：" + response + "</p><p>本邮件是系统自动发送，无需直接回复</p>");
            });
        } catch (Exception ex) {
            log.error("发送邮件异常", ex);
        }
    }

}
