package com.boco.nbd.wios.manage.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.flow.util.ProcessUtil;
import com.boco.nbd.wios.manage.contants.WiosConstant;
import com.boco.nbd.wios.manage.entity.bo.*;
import com.boco.nbd.wios.manage.mapper.def.DispatchConfigDao;
import com.boco.nbd.wios.manage.mapper.def.RegionDao;
import com.boco.nbd.wios.manage.mapper.def.SupplierContractDao;
import com.boco.nbd.wios.manage.mapper.def.SupplierDao;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 服务商service
 * 
 * @author xgl
 * @version [版本号, 2020年9月25日]
 */
@Service
public class SupplierService {
    @Autowired
    private SupplierDao supplierDao;
    
    @Autowired
    private AccountService accountService;
    
    @Autowired
    private RegionDao regionDao;
    @Autowired
    private RegionService regionService;
    
    @Autowired
    private SupplierContractDao supplierContractDao;
    
    @Autowired
    private DispatchConfigDao dispatchConfigDao;

    @Autowired
    private SapRemoteService sapRemoteService;

    
    /**
     * 添加服务商
     *
     * @param supplierBo
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void add(SupplierBo supplierBo) {
        if(supplierBo.getCode().length()<6){
            throw new ServiceException("编码长度至少6位");
        }
        if (checkExist(supplierBo.getCode(), supplierBo.getName(), null)) {
            throw new ServiceException("名称或编码已存在");
        }
        //判断sap是否存在，存在才添加
        if(sapRemoteService.verifyServiceProvider(supplierBo.getOrganizationCode()).getStatus()==-1){
            throw new ServiceException("sap系统中该服务商编码不存在");
        }
        doWhenAdd(supplierBo);
        supplierDao.insert(supplierBo);
    }
    
    public void doWhenAdd(SupplierBo supplierBo) {
        AccountBo account = new AccountBo();
        account.setAccount(supplierBo.getAccount());
        account.setName(supplierBo.getContactName());
        account.setPassword(WiosConstant.ACCOUNT_INIT_PASSWORD);
        account.setPhone(supplierBo.getAccount());
        // 默认服务商管理员
        account.setRoleIds("4");
        accountService.addAccount(account);
        supplierBo.setAccountId(account.getId());
    }
    
    /**
     * 批量添加
     *
     * @param supplierJson
     */
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false, rollbackFor = Exception.class)
    public void batchAdd(String supplierJson) {
        ObjectMapper mapper = new ObjectMapper();
        try {
            List<SupplierBo> list = mapper.readValue(supplierJson, new TypeReference<List<SupplierBo>>() {
            });
            List<Supplier> supplierList = new ArrayList<>();
            list.forEach(supplierBo -> {
                doWhenAdd(supplierBo);
                Supplier s = new Supplier();
                BeanUtils.copyProperties(supplierBo, s);
                supplierList.add(s);
            });
            supplierDao.batchInsert(supplierList);
        } catch (JsonProcessingException e) {
            throw new ServiceException("转换失败");
        }
    }

    public Supplier getById(Integer id){
        return supplierDao.selectById(id);
    }

    public List<SupplierVo> getList(SupplierBo condition) {
        SessionInfo sessionInfo = ProcessUtil.getUserInfo();
        if (sessionInfo != null && !StringUtils.isEmpty(sessionInfo.getAttribute().get("installSupplierId"))) {
            condition.setId(sessionInfo.getAttribute().get("installSupplierId"));
        }
        if (condition.getServiceRegionId() != null) {
            Region region = regionDao.selectById(condition.getServiceRegionId());
            List<String> regionIdList = Arrays.asList(region.getIdTree().split(CamsConstant.UNDER_LINE));
            List<SupplierContract> scList = supplierContractDao.selectByRegionId(regionIdList,null);
            List<String> supplierIdList = scList.stream().map(SupplierContract::getSupplierId).collect(Collectors.toList());
            condition.setIds(supplierIdList);
        }
        return supplierDao.getList(condition);
    }
    
    public void update(int id, SupplierBo supplierBo) {
        supplierDao.update(id, supplierBo);
    }
    
    /**
     * 更换账户
     *
     * @param accountId
     * @param account
     * @return
     */
    public int updateAccount(int accountId, String account) {
        AccountBo accountEntity = new AccountBo();
        accountEntity.setId(accountId);
        accountEntity.setAccount(account);
        accountEntity.setPhone(account);
        return accountService.updateAccount(accountEntity);
    }
    
    /**
     * 校验唯一性
     *
     * @param code
     * @param name
     * @return
     */
    public boolean checkExist(String code, String name, Integer id) {
        List<Supplier> list = supplierDao.selectOnly(code, name);
        if (id != null) {
            if (!list.isEmpty() && id == Integer.parseInt(list.get(0).getId())) {
                return false;
            }
        }
        return !list.isEmpty();
    }

    public Supplier selectByAccountId(Integer accountId){
        return supplierDao.selectByAccountId(accountId);
    }

    /**
     * @Description 审核新建的安装服务商
     * @Param [id, checkStatus, checkRemark]
     * @author liwenxiang
     * @date 2021/7/27 13:34
     * @return void
     */
    public void checkSupplier(Integer id, Integer checkStatus, String checkRemark) {
        Assert.notNull(id,"安装服务商id必填");
        Assert.notNull(checkStatus,"审核状态必填");
        Supplier supplier = supplierDao.selectById(id);
        if (ObjectUtils.isEmpty(supplier)) {
            throw new ServiceException("服务商不存在");
        }
        Supplier supplierCheck= new Supplier();
        supplierCheck.setId(id+"");
        supplierCheck.setCheckStatus(checkStatus);
        supplierCheck.setCheckRemark(checkRemark);
        supplierCheck.setModifyTime(new Date());
        supplierDao.updateDynamic(supplierCheck);
    }

    public List<RegionVo> getDispatchConfig(Integer id) {
        Assert.notNull(id,"安装服务商id必填");
        List<SupplierContract> supplierContractList=supplierContractDao.selectEffectiveContract(id.toString(), null, 2);
        if (CollectionUtils.isEmpty(supplierContractList)) {
            throw new ServiceException("服务商签约不存在");
        }
        SupplierContract supplierContract = supplierContractList.get(0);
        String[] regionIdSplit = supplierContract.getRegionId().split(CamsConstant.COMMA);
        List<Integer> regionIdList = new ArrayList<>();
        for (String s : regionIdSplit) {
            regionIdList.add(Integer.parseInt(s));
        }
        DispatchConfigCriteria exp = new DispatchConfigCriteria();
        exp.createCriteria().andSupplierIdEqualTo(id+"");
        List<DispatchConfig> dispatchConfigList = dispatchConfigDao.selectByExample(exp);
        Map<Integer, DispatchConfig> dispatchConfigMap = dispatchConfigList.stream().collect(Collectors.toMap(DispatchConfig::getRegionId, a -> a, (k1, k2) -> k2));

        List<Region> regionList = new ArrayList<>();
        for (Integer rid : regionIdList) {
            List<Region> sonList = regionDao.selectSelfAndChildren(rid);
            regionList.addAll(sonList);
        }
        return regionService.convertRegions(regionList,dispatchConfigMap);
    }

    public void updateDispatchConfig(String supplierId,String jsonStr) {
        JSONArray jsonArray = JSONUtil.parseArray(jsonStr);
        Date modifyTime = new Date();
        if(jsonArray.size()>0){
            for(int i = 0 ;i<jsonArray.size();i++){
                JSONObject jsonObject =jsonArray.getJSONObject(i);
                DispatchConfig dispatchConfig = new DispatchConfig();
                //判断id是否存在，不存在，则增加，存在则更新
                if (ObjectUtils.isEmpty(jsonObject.get("id"))) {
                    dispatchConfig.setId(UUID.randomUUID().toString());
                    dispatchConfig.setRegionId(jsonObject.getInt("regionId"));
                    dispatchConfig.setDispatchNum(jsonObject.getInt("dispatchNum"));
                    dispatchConfig.setSupplierId(supplierId);
                    dispatchConfig.setCreateTime(modifyTime);
                    dispatchConfig.setModifyTime(modifyTime);
                    dispatchConfigDao.insertSelective(dispatchConfig);
                }else{
                    dispatchConfig.setId(String.valueOf(jsonObject.get("id")));
                    dispatchConfig.setDispatchNum(jsonObject.getInt("dispatchNum"));
                    dispatchConfig.setModifyTime(modifyTime);
                    dispatchConfigDao.updateByPrimaryKeySelective(dispatchConfig);
                }
            }
            DispatchConfigCriteria exp = new DispatchConfigCriteria();
            exp.createCriteria().andSupplierIdEqualTo(supplierId).andModifyTimeLessThan(DateUtil.offsetSecond(modifyTime,-1));
            dispatchConfigDao.deleteByExample(exp);
        }
    }
}
