package com.boco.nbd.wios.wx.service;

import com.boco.nbd.cams.core.constant.CamsConstant;
import com.boco.nbd.wios.wx.weixin.WeixinPayTradeType;
import com.ihidea.component.pay.PayPropertySupport;
import com.ihidea.core.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;


/**
 * @author kevin
 */
@Component
@Slf4j
public class WeixinPayCore {

	private static String wiosApiUrl;

	@Value("${url.api.wios}")
	public void setWiosApiUrl(String wiosApiUrl){
		WeixinPayCore.wiosApiUrl = wiosApiUrl;
	}

	/**
	 * 得到签名
	 * @param params
	 * @return
	 */
	public static String getSign(Map<String, String> params) {

		// 参数排序
		String preSingStr = createLinkString(params);

		// 拼接API密钥
		preSingStr = preSingStr + "&key=" + PayPropertySupport.getProperty("pay.weixin.key");

		return DigitalUtils.byte2hex(SignatureUtils.md5(preSingStr));
	}

	private static String createLinkString(Map<String, String> params) {

		List<String> keys = new ArrayList<>(params.keySet());
		Collections.sort(keys);

		String prestr = "";

		for (int i = 0; i < keys.size(); i++) {
			String key = keys.get(i);
			String value = params.get(key);

			// 拼接时，不包括最后一个&字符
			if (i == keys.size() - 1) {
				prestr = prestr + key + "=" + value;
			} else {
				prestr = prestr + key + "=" + value + "&";
			}
		}

		return prestr;
	}

	/**
	 * 查询订单状态<br>
	 * SUCCESS—支付成功<br>
	 * REFUND—转入退款<br>
	 * NOTPAY—未支付<br>
	 * CLOSED—已关闭<br>
	 * REVOKED—已撤销<br>
	 * USERPAYING--用户支付中<br>
	 * NOPAY--未支付(输入密码或确认支付超时)<br>
	 * PAYERROR--支付失败(其他原因，如银行返回失败)<br>
	 * @param orderId
	 * @return
	 */
	public static Map<String, Object> queryOrder(String orderId) {

		Map<String, String> params = new HashMap<String, String>(8);

		// 公众账号ID
		params.put("appid", PayPropertySupport.getProperty("pay.weixin.appid"));

		// 商户号
		params.put("mch_id", PayPropertySupport.getProperty("pay.weixin.mchId"));

		// 商户订单号
		params.put("out_trade_no", orderId);

		// 随机字符串
		params.put("nonce_str", String.valueOf(System.currentTimeMillis() / 1000));

		// 签名
		params.put("sign", getSign(params));

		String resultJSON = HttpClientUtils.post("https://api.mch.weixin.qq.com/pay/orderquery",
				"<xml>" + XMLUtilsEx.serialize(params) + "</xml>", "UTF-8", "UTF-8");

		log.info("查询订单支付返回结果：{}",resultJSON);
		Map<String, Object> resultMap = XMLUtilsEx.deserialize(resultJSON, Map.class);

		if (resultMap.containsKey("trade_state")) {
			if(StringUtils.equals("SUCCESS", (String)resultMap.get("trade_state"))){
				return resultMap;
			}
		}
		return null;
	}

	/**
	 * 得到统一下单的PrepayId
	 * @return
	 */
	public static String getPayPrepayId(String orderId, String orderName, BigDecimal price, String ipAddress, String tradeType, String openId) {
		Map<String, Object> resultMap = payPrepay(orderId, orderName, price, ipAddress, tradeType, openId, null);
		return (String) resultMap.get("prepay_id");
	}

	/**
	 * 获取统一下单的code_url
	 * @param orderId
	 * @param orderName
	 * @param price
	 * @param ipAddress
	 * @return
	 */
	public static String getCodeUrl(String orderId, String orderName, BigDecimal price, String ipAddress, String productId) {
		Map<String, Object> resultMap = payPrepay(orderId, orderName, price, ipAddress, WeixinPayTradeType.NATIVE.toString(), null, productId);
		return (String) resultMap.get("code_url");
	}

	/**
	 * 生成统一订单
	 * 在微信支付中，同一商户下的商户订单号必须唯一，否则提示商户订单号重复，除非多次请求时参数不变，微信支付会当做同一个订单；
	 * 当前wios支持扫码支付（NATIVE）和公众号支付(JSAPI)，所以通过orderId + "_" + tradeType 进行区分，防止冲突，支付完成后的回调处理也按照相应方式反向处理
	 * @param openId
	 * @param orderId
	 * @param orderName
	 * @param price
	 * @param ipAddress
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private static Map<String, Object> payPrepay(String orderId, String orderName, BigDecimal price, String ipAddress, String tradeType, String openId, String productId) {

		Map<String, String> params = new HashMap<>(8);

		// 公众账号ID
		params.put("appid", PayPropertySupport.getProperty("pay.weixin.appid"));

		// 商户号
		params.put("mch_id", PayPropertySupport.getProperty("pay.weixin.mchId"));

		// 随机字符串
		params.put("nonce_str", String.valueOf(System.currentTimeMillis() / 1000));

		// 商品描述
		params.put("body", orderName);

		// 商户订单号
		params.put("out_trade_no", orderId+ CamsConstant.UNDER_LINE+tradeType);

		// 总金额(x100)
		params.put("total_fee", String.valueOf(price.multiply(BigDecimal.valueOf(100)).setScale(0, RoundingMode.UP)));

		// 终端IP
		params.put("spbill_create_ip", ipAddress);

		// 通知地址
		params.put("notify_url", wiosApiUrl + "/" +PayPropertySupport.getProperty("pay.weixin.notifyUrl"));

		// 交易类型
		params.put("trade_type", tradeType);

		if(StringUtils.equals(tradeType, WeixinPayTradeType.JSAPI.toString())){
			params.put("openid", openId);
			params.put("attach", "1");
		}else if(StringUtils.equals(tradeType, WeixinPayTradeType.NATIVE.toString())){
			params.put("product_id", productId);
			params.put("attach", "2");
		}

		// 签名
		params.put("sign", getSign(params));

		String paramsXml=XMLUtilsEx.serialize(params);
		log.info("统一下单参数:{}",paramsXml);
		
		String resultJSON = HttpClientUtils.post("https://api.mch.weixin.qq.com/pay/unifiedorder", "<xml>" + paramsXml
				+ "</xml>", "UTF-8", "UTF-8");
		log.info("统一下单返回:{}",resultJSON);
		Map<String, Object> resultMap = XMLUtilsEx.deserialize(resultJSON, Map.class);
		
		if (resultMap.containsKey("return_code")) {
			return resultMap;
		}
		return Collections.emptyMap();
	}

	public static Map<String, String> getJSPayParam(String orderId, String orderName, BigDecimal price, String ipAddress, String openId) {

		Map<String, String> payParam = new HashMap<>(8);
		payParam.put("appId", PayPropertySupport.getProperty("pay.weixin.appid"));
		
		String timestamp = String.valueOf((System.currentTimeMillis() / 1000));
		payParam.put("timeStamp", timestamp);
		payParam.put("nonceStr", StringUtilsEx.bytes2Hex(SignatureUtils.md5(timestamp)));
		
		log.info("openId:"+openId+",orderId:"+orderId+",orderName:"+orderName+",price:"+price+",ipAddress:"+ipAddress);
		String prepayId = getPayPrepayId(orderId, orderName, price, ipAddress, WeixinPayTradeType.JSAPI.toString(), openId);
		payParam.put("package", "prepay_id="+prepayId);
		payParam.put("signType", "MD5");
		payParam.put("paySign", getSign(payParam));
        log.info(payParam.toString());
		return payParam;
	}
}
